package misc;

/**
 * Title:        Mini Sharp C
 * Description:  Semestrln prce z pedmt Programovac jazyky a Programovac jazyky pro web
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Pavel Vank
 * @version 1.0
 */

/** Trida Lexan predstavuje vlastni lexikalni analyzator programu MiSC.
 *  Rozpoznava jednotlive lexikalni elementy na vstupu a predklada jejich ciselny kod na vystup.
 *  Zpracovava cisla v 8,10,16 -ove soustave, klicova slova, identifikatory a dalsi elementy.
 *  Trida Lexan je implementovana staticky a jeji hlavni metoda je getSymbo(), ktera
 *  na kazde zavolani vrati jeden kod znaku s textem lex. elementu v promenne Symbol.
 */

 public class Lexan {

  private static String[] keywords = {"do","for",
                                      "int","float",
                                      "if","else",
                                      "while",
                                      "printf",
                                      "void"
                                      };

  public static final int TECKA =  1;	//.
  public static final int CARKA =  2;	//,
  public static final int STRED =  3;	//;
  public static final int LZAV  =  4;	//(
  public static final int PZAV  =  5;	//)
  public static final int LOMEN =  6;	///
  public static final int KRAT  =  7;	//*
  public static final int MINUS =  8;	//-
  public static final int PLUS  =  9;	//+
  public static final int ROVNO = 10;	//=
  public static final int MENSI = 11;	//<
  public static final int VETSI = 12;	//>

  public static final int JERO  = 13;	//==
  public static final int MENRO = 14;	//<=
  public static final int VETRO = 15;	//>=
  public static final int NERO  = 16;	//!=
  public static final int PLURO = 17;	//+=
  public static final int MINRO = 18;	//-=
  public static final int KRARO = 19;	//*=
  public static final int LOMRO = 20;	///=
  public static final int MINMI = 21;	//--
  public static final int PLUPL = 22;	//++
  public static final int KOME1 = 23;	////
  public static final int KOMST = 24;	///*
  public static final int KOMKO = 25;	//*/

  public static final int HEXA  = 26;	//x
  public static final int BEGIN = 27;	//{
  public static final int END   = 28;	//}
  public static final int MODUL = 29;	//%
  public static final int NEGAC = 30;	//!
  public static final int XOR   = 31;	//^
  public static final int DOPLN = 32;	//~
  public static final int OR    = 33;	//|
  public static final int AND   = 34;	//&
  public static final int HRZA1 = 35;	//[
  public static final int HRZA2 = 36;	//]
  public static final int TERNA = 37;	//?
  public static final int DVOJT = 38;	//:
  public static final int BACKS = 39;	//\
  public static final int UVOZO = 40;	//"
  public static final int APOST = 41;	//'

  public static final int KWDO  = 100;	//do
  public static final int KWFOR = 101;	//for
  public static final int KWINT = 102;	//int
  public static final int KWFLO = 103;	//float
  public static final int KWIF  = 104;	//if
  public static final int KWELS = 105;	//else
  public static final int KWWHI = 106;	//while
  public static final int KWPRI = 107;	//printf
  public static final int KWVOI = 108;	//void

  public static final int IDENT = 200;	//Identifikator
  public static final int CISLO = 300;	//Cislo
  public static final int STRING= 400;  //String + uvozovky

/** Nacteny symbol ze vstupniho souboru.
 *  Cisla v nem jsou ulozena dle vstupniho tvaru = dekadicky, oktalove nebo hexadwcimalne.
 */
  public static String symbol;                                                 //Precteny symbol

/** Zaklad cisla uvedeneho  v promenne Symbol.
 *  Nabyva hodnot 8,10,16.
 */
  public static int zaklad;                                                    //Ciselny zaklad soustavy

  private static int kwindex;                                                  //Cislo klicoveho slova
  private static char piece;                                                   //Predem nacteny znak

/*****************************************************************************/

  private static boolean isPrazdny(char pismeno) {                             //Vyfiltruje bile znaky
  if ((pismeno=='\t') | (pismeno=='\n') | (pismeno=='\r')| (pismeno==' ')) return true;
    	else return false;
  }

/*---------------------------------------------------------------------------*/

  private static boolean isPisOrCis(char pismeno) {
  if (((pismeno>='A') && (pismeno<='Z')) || ((pismeno>='a') && (pismeno<='z')) ||
      ((pismeno>='0') && (pismeno<='9')))
  	return true;
  		else return false;
  }
/*---------------------------------------------------------------------------*/

 private static boolean isCislo(char pismeno) {                                //Jestli to je cislo nebo .
 if ((Character.isDigit(pismeno)) | (pismeno=='.')) return true;
  return false;
 }

/*---------------------------------------------------------------------------*/

 private static boolean isHexa(char pismeno) {
 char pom=Character.toUpperCase(pismeno);
 if ((Character.isDigit(pismeno)) | ((pom>='A') & (pom<='F'))) return true;
   return false;
 }

/*---------------------------------------------------------------------------*/

 private static boolean isOkta(char pismeno) {
 if ((Character.isDigit(pismeno)) & ((pismeno>='0') & (pismeno<='7'))) return true;
  return false;
 }

/*---------------------------------------------------------------------------*/
  private static boolean isKeyword(String slovo) {                             //Je to keyword?
    int i=0;
    while(i<keywords.length) if (keywords[i++].equals(slovo)) {
                                kwindex=(i-1)+100;
                                return true;
                              }
     return false;
  }

/*---------------------------------------------------------------------------*/
/** Startovaci metoda, ktera provede nastaveni vstupniho souboru a nacte prvni znak.
 *  Obaluje tak metodu Sys.ctiZnak().
 */
  public static void init(String soubor) {                                     //Priradi soubor a nacte znak
   Sys.priradSoubor(soubor);
   piece=Sys.ctiZnak();
  }

/*---------------------------------------------------------------------------*/
/** Vlastni cteni symbolu.
 *  Preskakuje bile znaky jako jsou mezery, tabulatory, CR a LF.
 *  Vraci ciselny kod lexikalniho elementu a pres promennou Symbol
 *  jeho textovou formu.
 */
  public static int getSymbol() throws Err{                                    //Vrati kod symbolu a ulozi ho do prom.
    while (isPrazdny(piece)) piece=Sys.ctiZnak();                              //Preskocime bile znaky
    switch(piece) {
        case '.': symbol=String.valueOf(piece); piece=Sys.ctiZnak();return TECKA;
	case ',': symbol=String.valueOf(piece); piece=Sys.ctiZnak();return CARKA;
	case ';': symbol=String.valueOf(piece); piece=Sys.ctiZnak();return STRED;
	case '(': symbol=String.valueOf(piece); piece=Sys.ctiZnak();return LZAV;
	case ')': symbol=String.valueOf(piece); piece=Sys.ctiZnak();return PZAV;
	case '/': symbol=String.valueOf(piece); piece=Sys.ctiZnak();
                  if (piece=='=') {symbol+=String.valueOf(piece);piece=Sys.ctiZnak();return LOMRO;}
                  if (piece=='/') {symbol+=String.valueOf(piece);piece=Sys.ctiZnak();return KOME1;}
                  if (piece=='*') {symbol+=String.valueOf(piece);piece=Sys.ctiZnak();return KOMST;}
                   return LOMEN;
	case '*': symbol=String.valueOf(piece); piece=Sys.ctiZnak();
                  if (piece=='=') {symbol+=String.valueOf(piece);piece=Sys.ctiZnak();return KRARO;}
                  if (piece=='/') {symbol+=String.valueOf(piece);piece=Sys.ctiZnak();return KOMKO;}
                   return KRAT;
	case '-': symbol=String.valueOf(piece); piece=Sys.ctiZnak();
                  if (piece=='=') {symbol+=String.valueOf(piece);piece=Sys.ctiZnak();return MINRO;}
                  if (piece=='-') {symbol+=String.valueOf(piece);piece=Sys.ctiZnak();return MINMI;}
                   return MINUS;
	case '+': symbol=String.valueOf(piece); piece=Sys.ctiZnak();
                  if (piece=='=') {symbol+=String.valueOf(piece);piece=Sys.ctiZnak();return PLURO;}
                  if (piece=='+') {symbol+=String.valueOf(piece);piece=Sys.ctiZnak();return PLUPL;}
                   return PLUS;
	case '=': symbol=String.valueOf(piece); piece=Sys.ctiZnak();
                  if (piece=='=') {symbol+=String.valueOf(piece);piece=Sys.ctiZnak();return JERO;}
                   return ROVNO;
	case '<': symbol=String.valueOf(piece); piece=Sys.ctiZnak();
                  if (piece=='=') {symbol+=String.valueOf(piece);piece=Sys.ctiZnak();return MENRO;}
                   return MENSI;
        case '>': symbol=String.valueOf(piece); piece=Sys.ctiZnak();
                  if (piece=='=') {symbol+=String.valueOf(piece);piece=Sys.ctiZnak();return VETRO;}
                   return VETSI;
        case '{': symbol=String.valueOf(piece); piece=Sys.ctiZnak();return BEGIN;
        case '}': symbol=String.valueOf(piece); piece=Sys.ctiZnak();return END;
        case '%': symbol=String.valueOf(piece); piece=Sys.ctiZnak();return MODUL;
        case '!': symbol=String.valueOf(piece); piece=Sys.ctiZnak();
                  if (piece=='=') {piece=Sys.ctiZnak();return NERO;} return NEGAC;
        case '^': symbol=String.valueOf(piece); piece=Sys.ctiZnak();return XOR;
        case '~': symbol=String.valueOf(piece); piece=Sys.ctiZnak();return DOPLN;
        case '|': symbol=String.valueOf(piece); piece=Sys.ctiZnak();return OR;
        case '&': symbol=String.valueOf(piece); piece=Sys.ctiZnak();return AND;
        case '[': symbol=String.valueOf(piece); piece=Sys.ctiZnak();return HRZA1;
        case ']': symbol=String.valueOf(piece); piece=Sys.ctiZnak();return HRZA2;
        case '?': symbol=String.valueOf(piece); piece=Sys.ctiZnak();return TERNA;
        case ':': symbol=String.valueOf(piece); piece=Sys.ctiZnak();return DVOJT;
        case '\\': symbol=String.valueOf(piece); piece=Sys.ctiZnak();return BACKS;
        case '\'': symbol=String.valueOf(piece); piece=Sys.ctiZnak();return APOST;
        case '"': symbol=String.valueOf(piece);                                //STRING
                   piece=Sys.ctiZnak();
                    while ((piece!='"') & (piece!='\n')) {                   //Konci " nebo \n
                     symbol+=piece;
                     piece=Sys.ctiZnak();
                    }
                    if (piece=='"') symbol+=piece;
                     else Syntan.chyba("Unfinished string");                   //Spravne je jen "
                     piece=Sys.ctiZnak();
                  return STRING;

        case (char)-1 : return (-1); // konec vstupu

        default:
         if (Character.isDigit(piece)) {                                       //CISLO
          if (piece=='0') {                                                    //Hexa nebo okta nbo 0
           if ((piece=Character.toUpperCase(Sys.ctiZnak()))=='X') {            //HEXA
              zaklad=16; piece=Sys.ctiZnak();
              if (isHexa(piece)) {symbol=String.valueOf(piece);
                piece=Sys.ctiZnak();} else {Syntan.chyba("Wrong hexa number");return 300;} //Bylo zadano jen 0x
                while(isHexa(piece)) {
	                symbol+=String.valueOf(piece);piece=Sys.ctiZnak();
                } return 300;
           } //HEXA
            else {                                                             //Okta nebo 0
             if (Character.isDigit(piece)) {                     //OKTA
                zaklad=8; if(!isOkta(piece)) Syntan.chyba("Wrong octa number");//Spatne okta cislo za nulou
                symbol=String.valueOf(piece);piece=Sys.ctiZnak();
                while(isOkta(piece)) {
	                symbol+=String.valueOf(piece);piece=Sys.ctiZnak();
                } return 300;
             } //OKTA
              else { zaklad=10; symbol=String.valueOf('0');}         //Je to 0
            } //else
          } //hexa or okta
           else {                                                              //Desitkove
            symbol=String.valueOf(piece); piece=Sys.ctiZnak();                 //Ulozime 1. znak cisla
            zaklad=10;
           }
            while(isCislo(piece)) {
              symbol+=String.valueOf(piece);piece=Sys.ctiZnak();
            }
        return 300;

         } //if
          else {                                                               //KEYWORD nebo IDENT
            symbol=String.valueOf(piece); piece=Sys.ctiZnak();                 //Ulozime 1. znak keywordu nebo identu
            while(isPisOrCis(piece)) {                                         //Dokud neni whitespace
              symbol+=piece; piece=Sys.ctiZnak();                              //Precteme dalsi znak
           } //while
           if (isKeyword(symbol)) {
            return kwindex;
           } //if
            else {
             return 200;
            } //else
          } //else
    } //switch
  } //getSymbol

/*---------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------*/

} //Lexan