package misc;

/**
 * Title:        Mini Sharp C
 * Description:  Semestrln prce z pedmt Programovac jazyky a Programovac jazyky pro web
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Pavel Vank
 * @version 1.0
 */

class Err extends Exception {                                                 //Vyjimka pro chybu
  public Err() {
    super("File "+Gui.soubor+": Cannot compile source file because of errors");
  }
}

/** Trida Syntan je implementaci vlastniho syntaktickeho analyzatoru programu MiSC.
 *  Obsahuje rekurzivni sestup dle atributove prekladove gramatiky.
 */


public class Syntan {

/** Vnitrni forma programu v C.
 *  Binarni strom s uzly odpovidajicimi jednotlivym operacim.
 */

public static Program strom;                                                  //Vnitrni forma

private static int kod;                                                       //Kod symbolu

/*****************************************************************************/

/** Metoda pro vypisovani chyby ve zdrojovem textu.
 *  Je zde explicitne volana vyjimka Err.
 */

public static void chyba(String s) throws Err {                               //Chybove hlasky
  Gui.txt.append("File "+Gui.soubor+": Syntax error occured at line "+Sys.radek+" "+s+"\n");
  throw new Err();
}

/*---------------------------------------------------------------------------*/

private static void vypis(String text) {
  System.out.println("- "+text);
}

/*---------------------------------------------------------------------------*/

private static void srovnejIdent() throws Err {                                //Na rade ma byt IDENT S kontrolou
  if (kod!=Lexan.IDENT) chyba("Identifier expected");
  if (!Kernel.promenne.containsKey(Lexan.symbol)) chyba("Identifier "+Lexan.symbol+" unknown");
}

/*---------------------------------------------------------------------------*/

private static void srovnejIdent(int a) throws Err {                           //Na rade IDENT BEZ kontroly
  if (kod!=Lexan.IDENT) chyba("Identifier expected");
}

/*---------------------------------------------------------------------------*/

private static boolean specRovno() throws Err {                                           //Testuje druhy rovnitek
 switch (kod) {
 case Lexan.ROVNO :
 case Lexan.PLURO :
 case Lexan.MINRO :
 case Lexan.KRARO :
 case Lexan.LOMRO : ctiSymbol(); return true;
 default : return false;
 }
}

/*---------------------------------------------------------------------------*/

private static void ctiSymbol() throws Err {                                   //Obalujici metoda Lexanu
  kod=Lexan.getSymbol();
  if (kod==Lexan.KOMST) {                                                      //Preskocime komentare
    while(kod!=Lexan.KOMKO)
      kod=Lexan.getSymbol();
    kod=Lexan.getSymbol();
  }
}

/*---------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------*/

private static Program _program() throws Err {

  switch (kod) {
  case Lexan.KWVOI : return new Program(_funkce());                    //Program->Funkce Program
                     //_program();
                     //break;
  case Lexan.KWINT :                                                   //Program->Deklarace Funkce Program
  case Lexan.KWFLO : _deklarace();
                     return new Program(_funkce());
                     //_program();
                     //break;
  //default: ;                                                         //Program->epsilon
  default : chyba("Wrong start of program");return null;
  }
}

/*---------------------------------------------------------------------------*/

private static void _deklarace() throws Err {                         //Deklarace->Typ ident D2
  _typ();
  srovnejIdent(1);                                                    //Bez kontroly
  /* out */ Kernel.putProm(Lexan.symbol);
  ctiSymbol();_d2();
}

/*---------------------------------------------------------------------------*/

private static void _d2() throws Err {                               //D2->D3 | =VyrazD3
  switch(kod) {
  case Lexan.STRED :
  case Lexan.CARKA : _d3();
                     break;
  case Lexan.ROVNO : ctiSymbol(); if ((kod==Lexan.CISLO) | (kod==Lexan.IDENT)) ctiSymbol();
                                    else chyba("Identifier or number expected");
                     _d3();
                     break;
  default : chyba("Wrong declaration");
  }
}

/*---------------------------------------------------------------------------*/

private static void _d3() throws Err {                               //D3->;D4 | ,ident D2
  switch(kod) {
  case Lexan.STRED : ctiSymbol();_d4();
                     break;
  case Lexan.CARKA : ctiSymbol();
                     srovnejIdent(1);                                         //Bez kontroly
                     /* out */ Kernel.putProm(Lexan.symbol);
                     ctiSymbol();_d2();
                     break;
  default : chyba("Unfinished declaration");
  }
}

/*---------------------------------------------------------------------------*/

private static void _d4() throws Err {                               //D4->Deklarace | epsilon
  switch(kod) {
  case Lexan.KWINT :
  case Lexan.KWFLO : _deklarace();
                     break;
  default : ;
  }
}

/*---------------------------------------------------------------------------*/

private static void _typ() throws Err {                              //Typ->int | float
  switch(kod) {
  case Lexan.KWINT : break;
  //case Lexan.KWFLO : break;
  default : chyba("Unknown type in declaration");
  }
 ctiSymbol();
}

/*---------------------------------------------------------------------------*/

private static Vyraz _vyraz() throws Err {                            //Vyraz->TE'
  return _ec(_t());
}

/*---------------------------------------------------------------------------*/

private static Vyraz _ec(Vyraz pom) throws Err {                               //E'->+TE'| -TE' | epsilon
  switch(kod) {                       //OK
  case Lexan.PLUS : ctiSymbol();
                    return _ec(new BinOp(Lexan.PLUS,pom,_t()));
                    //break;
  case Lexan.MINUS: ctiSymbol();
                    return _ec(new BinOp(Lexan.MINUS,pom,_t()));
                    //break;
  default : return pom;
  }
}

/*---------------------------------------------------------------------------*/

private static Vyraz _t() throws Err {                                //T->FT'
  return _tc(_f());
}

/*---------------------------------------------------------------------------*/

private static Vyraz _tc(Vyraz pom) throws Err {                               //T'->*FT' | /FT' | epsilon
  switch(kod) {                         //OK
  case Lexan.KRAT : ctiSymbol();
                    return _tc(new BinOp(Lexan.KRAT,pom,_f()));
                    //break;
  case Lexan.LOMEN: ctiSymbol();
                    return _tc(new BinOp(Lexan.KRAT,pom,_f()));
                    //break;
  default : return pom;
  }
}

/*---------------------------------------------------------------------------*/

private static Vyraz _f() throws Err {                                 //F->ident | cislo | (Vyraz)
  Vyraz pom;
  int cislo;

  switch(kod) {                           //OK
  case Lexan.IDENT : srovnejIdent(); pom=new Ident(Kernel.getAdresa(Lexan.symbol));
                     ctiSymbol(); _postIdent();
                     return pom;
                     //break;
  case Lexan.CISLO : cislo=Integer.valueOf(Lexan.symbol,Lexan.zaklad).intValue();
                     if (cislo>0xFF) chyba("Value too high");                  //Vetsi jak byte
                     pom= new Cislo(cislo);ctiSymbol();
                     return pom;
                     //break;
  case Lexan.LZAV  : ctiSymbol();pom=_vyraz();
                     if (kod!=Lexan.PZAV) chyba(") expected");
                     ctiSymbol();
                     return pom;
                     //break;
  default : chyba("Wrong expression");return null;
  }
}

/*---------------------------------------------------------------------------*/

private static Funkce _funkce() throws Err {                          //Funkce->void ident() {Prikaz}
  Seznam pom;

  ctiSymbol(); srovnejIdent(1);                           //Bez kontroly
  ctiSymbol(); if (kod!=Lexan.LZAV) chyba("( expected");
  ctiSymbol(); if (kod!=Lexan.PZAV) chyba(") expected");
  ctiSymbol();
  pom=_slozPrikaz();
  return new Funkce(0x1200,pom);
}

/*---------------------------------------------------------------------------*/

private static Prikaz _prikaz() throws Err {
  Prikaz pom;
  Vyraz pom2;

  switch(kod) {
  case Lexan.STRED : ctiSymbol(); return new EmptyP();
                     //break;
  case Lexan.BEGIN : return _slozPrikaz();
                     //break;
  case Lexan.IDENT : pom=_prirazeni();
                     if (kod!=Lexan.STRED) chyba("; expected");
                     ctiSymbol();return pom;
                     //break;
  case Lexan.KWPRI : ctiSymbol();if (kod!=Lexan.LZAV) chyba("( expected");
                     ctiSymbol();
                     return _print2();
                     //break;
  case Lexan.KWIF  : ctiSymbol();if (kod!=Lexan.LZAV) chyba("( expected");
                     ctiSymbol();pom2=_podm();
                     if (kod!=Lexan.PZAV) chyba(") expected");
                     ctiSymbol(); return new If(pom2,_prikaz(),_if2());
                     //break;
  case Lexan.KWWHI : ctiSymbol();if (kod!=Lexan.LZAV) chyba("( expected");
                     ctiSymbol(); pom2=_podm();
                     if (kod!=Lexan.PZAV) chyba(") expected");
                     ctiSymbol(); return new While(pom2,_prikaz());
                     //break;
  case Lexan.KWFOR : ctiSymbol(); if (kod!=Lexan.LZAV) chyba("( expected");
                     ctiSymbol(); pom=_prirazeni();
                     if (kod!=Lexan.STRED) chyba("; expected");
                     ctiSymbol();return _for2(pom);
                     //break;
  default : ;
  } return null;
}

/*---------------------------------------------------------------------------*/

private static Seznam _slozPrikaz() throws Err {
 Prikaz pom,pom2;
                     if (kod!=Lexan.BEGIN) chyba("{ expected");
                     ctiSymbol();
                     pom=_prikaz(); pom2=_sloz2();
                     if (kod!=Lexan.END) chyba("} expected");
                     ctiSymbol();return new Seznam(pom,pom2);
}

/*---------------------------------------------------------------------------*/

private static Prikaz _sloz2() throws Err {
  Prikaz pom;
  switch(kod) {
  case Lexan.STRED :
  case Lexan.BEGIN :
  case Lexan.IDENT :
  case Lexan.KWPRI :
  case Lexan.KWIF  :
  case Lexan.KWWHI :
  case Lexan.KWFOR : pom=_prikaz();return new Seznam(pom,_sloz2());
  default : return new EmptyP();
  }
}

/*---------------------------------------------------------------------------*/

private static Prikaz _print2() throws Err {
 Vyraz pom2;
 String pom;
 switch (kod) {
  case Lexan.IDENT  : srovnejIdent(); pom2=new Ident(Kernel.getAdresa(Lexan.symbol));
                      ctiSymbol();_postIdent();
                      if (kod!=Lexan.PZAV) chyba(") expected");
                      ctiSymbol();if (kod!=Lexan.STRED) chyba("; expected");
                      ctiSymbol();
                      return new Print(pom2);
  case Lexan.STRING : pom=Lexan.symbol; ctiSymbol();
                      if (kod!=Lexan.PZAV) chyba(") expected");
                      ctiSymbol();if (kod!=Lexan.STRED) chyba("; expected");
                      ctiSymbol();
                      return new PrintS(pom);
  default : chyba("Wrong printf parameters"); return null;
 }

}

/*---------------------------------------------------------------------------*/

private static Prirazeni _prirazeni() throws Err {
  Ident pom;
  srovnejIdent(); pom=new Ident(Kernel.getAdresa(Lexan.symbol));
  ctiSymbol();
  if (!_postIdent()) {                                                        //i++ je take prirazeni!!!
                      if (kod!=Lexan.ROVNO) chyba("= expected");              //postIdent neni, musi byt =Vyraz
                      ctiSymbol();return new Prirazeni(pom,_vyraz());
  } else if (specRovno()) {                                                   //postIdent je, muze byt =Vyraz
                                return new Prirazeni(pom,_vyraz());
         }
  return new Prirazeni(pom,new EmptyV());
}

/*---------------------------------------------------------------------------*/

private static Prikaz _if2() throws Err {                             //If2->else Prikaz | epsilon
  if (kod==Lexan.KWELS) {
                         ctiSymbol();return _prikaz();
  }
  return new EmptyP();
}

/*---------------------------------------------------------------------------*/

private static BinOp _podm() throws Err {                           //Podm->ident Podm2 | cislo
  Vyraz pom;

  switch (kod) {
  case Lexan.IDENT : pom=new Ident(Kernel.getAdresa(Lexan.symbol));ctiSymbol(); _postIdent();
                     return new BinOp(kod,pom,_podm2());
                     //break;
  case Lexan.CISLO : pom=new Cislo(Integer.valueOf(Lexan.symbol).intValue());ctiSymbol();
                     return new BinOp(0,pom,new EmptyV());
                     //break;
  default : chyba("Identifier or number expected"); return null;
  }
}

/*---------------------------------------------------------------------------*/

private static Vyraz _podm2() throws Err {                        //Podm2->Porovnani Vyraz | epsilon
  switch(kod) {
  case Lexan.JERO  :
  case Lexan.NERO  :
  case Lexan.VETRO :
  case Lexan.VETSI :
  case Lexan.MENSI :
  case Lexan.MENRO : _porovnani();
                     return _vyraz();
                     //break;
  default : return null;
  }
}

/*---------------------------------------------------------------------------*/

private static void _porovnani() throws Err {
  switch(kod) {
  case Lexan.JERO  :
  case Lexan.NERO  :
  case Lexan.VETRO :
  case Lexan.VETSI :
  case Lexan.MENSI :
  case Lexan.MENRO : break;
  default : chyba("Porovnani expected");
  }
 ctiSymbol();
}

private static For _for2(Prikaz p1) throws Err {
  Vyraz pom;
  switch(kod) {
  case Lexan.STRED : ctiSymbol(); return _for3(p1,new EmptyV());
                     //break;
  case Lexan.IDENT : srovnejIdent();pom=_podm();
                     if (kod!=Lexan.STRED) chyba("; expected");
                     ctiSymbol(); return _for3(p1,pom);
                     //break;
  default: chyba("Wrong condition in for loop"); return null;
  }
}

private static For _for3(Prikaz p1,Vyraz p2) throws Err {
  Prikaz pom;
  switch(kod) {
  case Lexan.PZAV  : ctiSymbol(); return new For(p1,p2,new EmptyP(),_prikaz());
                     //break;
  case Lexan.IDENT : pom=_prirazeni(); if (kod!=Lexan.PZAV) chyba(") expected");
                     ctiSymbol(); return new For(p1,p2,pom,_prikaz());
                     //break;
  default : chyba("Wrong iteration in for loop"); return null;
  }
}

/*---------------------------------------------------------------------------*/

private static boolean _postIdent() throws Err {
 switch(kod) {
 case Lexan.MINMI :
 case Lexan.PLUPL : ctiSymbol();
                    return true;
 default: return false;
 }
}

/*---------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------*/

/** Startovaci metoda syntaktickeho analyzatoru.
 *  Precte dopredu jeden lexikalni symbol a do promenne strom priradi vnitrni formu
 *  programu v C
 */

public static void start() throws Err {
  ctiSymbol();
  strom=_program();
}

} //Syntan