package misc;

/**
 * Title:        Mini Sharp C
 * Description:  Semestrln prce z pedmt Programovac jazyky a Programovac jazyky pro web
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Pavel Vank
 * @version 1.0
 */

/*****************************************************************************/

public abstract class Uzel {
  public abstract Uzel optimizuj();
  public abstract String preloz();
}

/*****************************************************************************/

class Funkce extends Uzel {
  int adresa;
  Seznam seznam;

  public Funkce(int adresa, Seznam seznam) {
    this.seznam=seznam;
    this.adresa=adresa;
  }

  public Uzel optimizuj() {return this;}
  public String preloz() {
    seznam.preloz();
    return "";
  }
}

/*****************************************************************************/

class Program extends Uzel {
  Funkce prvni;

  public Program(Funkce prvni) {
    this.prvni=prvni;
  }

  public Uzel optimizuj() {return this;}
  public String preloz() {
    prvni.preloz();
    return "";
  }
}

/*****************************************************************************/

abstract class Vyraz extends Uzel {
}

/*****************************************************************************/

abstract class Prikaz extends Uzel {
}

/*****************************************************************************/

class Ident extends Vyraz {
  int adresa;

  public Ident(int adresa) {
    this.adresa=adresa;
  }

  public Uzel optimizuj() {return this;}
  public String preloz() {
    Start.sharp.pushVia(adresa);
    return "";
  }
}

/*****************************************************************************/

class BinOp extends Vyraz {
  public int operace;
  Vyraz levy,pravy;

  public BinOp(int operace, Vyraz levy, Vyraz pravy) {
  this.operace=operace;
  this.levy=levy;
  this.pravy=pravy;
  }

  public Uzel optimizuj() {return this;}
  public String preloz() {
    levy.preloz();
    pravy.preloz();
    Start.sharp.operace(operace);
    return Integer.toString(operace);
  }
}

/*****************************************************************************/

class Cislo extends Vyraz {
  int hodnota;

  public Cislo(int hodnota) {
    this.hodnota=hodnota;
  }

  public Uzel optimizuj() {return this;}
  public String preloz() {
    Start.sharp.push(hodnota);
    return "";
  }
}

/*****************************************************************************/

class UnMinus extends Vyraz {
  Vyraz vyraz;

  public UnMinus(Vyraz vyraz) {
    this.vyraz=vyraz;
  }

  public Uzel optimizuj() {return this;}
  public String preloz() {return "";}
}

/*****************************************************************************/

class EmptyV extends Vyraz {

  public Uzel optimizuj() {return this;}
  public String preloz()  {return "";}

}

/*****************************************************************************/

class EmptyP extends Prikaz {

  public Uzel optimizuj() {return this;}
  public String preloz()  {return "";}

}

/*****************************************************************************/

class PrintS extends Prikaz {
  String vyraz;

  public PrintS(String vyraz) {
    this.vyraz=vyraz;
  }

  public Uzel optimizuj() {return this;}
  public String preloz() {
   Start.sharp.printf(vyraz);
    return "";}
}

/*****************************************************************************/

class Print extends Prikaz {
  Vyraz vyraz;

  public Print(Vyraz vyraz) {
    this.vyraz=vyraz;
  }

  public Uzel optimizuj() {return this;}
  public String preloz() {
   vyraz.preloz();
   Start.sharp.printf();
    return "";}
}

/*****************************************************************************/

class If extends Prikaz {

  int pom,pom2,pom3,pom4;

  Vyraz podminka;
  Prikaz thenP,elseP;
  String oper;

  public If(Vyraz podminka, Prikaz thenP, Prikaz elseP) {
    this.podminka=podminka;
    this.thenP=thenP;
    this.elseP=elseP;
  }

  public Uzel optimizuj() {return this;}
  public String preloz() {

    oper=podminka.preloz();

    pom2=(Start.sharp.pozice-2); //Pozice v programu, kam se vepise adresa skoku

    thenP.preloz();

    if (elseP!=null) {
      Start.sharp.jump(pom3);       // JP pozice naslepo
      pom=Start.sharp.pozice-2;

      pom4=Start.sharp.pozice;      // Adresa, kdyz neni splnena podminka
      Start.sharp.program[pom2]=(byte)(Start.sharp.recountAdr(pom4)%0x100); //Vepsani adresy pro nespl.
      Start.sharp.program[pom2+1]=(byte)(Start.sharp.recountAdr(pom4)/0x100);

      if (oper==Integer.toString(Lexan.MENSI)) {
        Start.sharp.program[(pom2)-3]=(byte)(Start.sharp.recountAdr(pom4)%0x100); //Vepsani adresy pro nespl.
        Start.sharp.program[(pom2)-2]=(byte)(Start.sharp.recountAdr(pom4)/0x100);
      }

      elseP.preloz();
      pom3=Start.sharp.pozice;
      Start.sharp.program[pom]=(byte)(Start.sharp.recountAdr(pom3)%0x100); //Vepsani adresy pro preskoceni else
      Start.sharp.program[pom]=(byte)(Start.sharp.recountAdr(pom3)/0x100);
    } else {
        pom=Start.sharp.pozice;      // Adresa, kdyz neni splnena podminka
        Start.sharp.program[pom2]=(byte)(Start.sharp.recountAdr(pom)%0x100); //Vepsani adresy pro nespl.
        Start.sharp.program[pom2+1]=(byte)(Start.sharp.recountAdr(pom)/0x100);

        if (oper==Integer.toString(Lexan.MENSI)) {
          Start.sharp.program[(pom2)-3]=(byte)(Start.sharp.recountAdr(pom)%0x100); //Vepsani adresy pro nespl.
          Start.sharp.program[(pom2)-2]=(byte)(Start.sharp.recountAdr(pom)/0x100);
        }

      }

    return "";
  }
}

/*****************************************************************************/

class Prirazeni extends Prikaz {
  Ident ident;
  Vyraz vyraz;

  public Prirazeni(Ident ident, Vyraz vyraz) {
    this.ident=ident;
    this.vyraz=vyraz;
  }

  public Uzel optimizuj() {return this;}
  public String preloz() {
   vyraz.preloz();           //Vyhodi na zasobnik hodnotu vyrazu
   Start.sharp.assign(ident.adresa);    //Adresa identu
    return "";
  }
}

/*****************************************************************************/

class While extends Prikaz {
  Vyraz podminka;
  Prikaz prikaz;

  int pom,pom2;
  String oper;

  public While(Vyraz podminka, Prikaz prikaz) {
    this.podminka=podminka;
    this.prikaz=prikaz;
  }

  public Uzel optimizuj() {return this;}
  public String preloz() {

    pom=Start.sharp.pozice;      // Ulozeni pozice navratu

    oper=podminka.preloz();
    pom2=(Start.sharp.pozice-2); //Pozice v programu, kam se vepise adresa skoku
    prikaz.preloz();

    Start.sharp.jump(pom);       // JP adresa
    pom=Start.sharp.pozice;      // Adresa, kdyz neni splnena podminka
    Start.sharp.program[pom2]=(byte)(Start.sharp.recountAdr(pom)%0x100); //Vepsani adresy pro nespl.
    Start.sharp.program[(pom2)+1]=(byte)(Start.sharp.recountAdr(pom)/0x100);

    if (oper==Integer.toString(Lexan.MENSI)) {
      Start.sharp.program[(pom2)-3]=(byte)(Start.sharp.recountAdr(pom)%0x100); //Vepsani adresy pro nespl.
      Start.sharp.program[(pom2)-2]=(byte)(Start.sharp.recountAdr(pom)/0x100);
    }
    return "";
  }
}

/*****************************************************************************/

class For extends Prikaz {

  int pom,pom2;
  String oper;

  Vyraz p2;
  Prikaz p1,p3;
  Prikaz p4;

  public For(Prikaz p1, Vyraz p2, Prikaz p3, Prikaz p4) {
    this.p1=p1;
    this.p2=p2;
    this.p3=p3;
    this.p4=p4;
  }

  public Uzel optimizuj() {return this;}
  public String preloz() {
    p1.preloz();            //inicializace

    pom=Start.sharp.pozice;

    oper=p2.preloz();            //podminka

    pom2=(Start.sharp.pozice-2); //Pozice v programu, kam se vepise adresa skoku

    p4.preloz();
    p3.preloz();

    Start.sharp.jump(pom);       // JP adresa
    pom=Start.sharp.pozice;      // Adresa, kdyz neni splnena podminka
    Start.sharp.program[pom2]=(byte)(Start.sharp.recountAdr(pom)%0x100); //Vepsani adresy pro nespl.
    Start.sharp.program[pom2+1]=(byte)(Start.sharp.recountAdr(pom)/0x100);

    if (oper==Integer.toString(Lexan.MENSI)) {
      Start.sharp.program[(pom2)-3]=(byte)(Start.sharp.recountAdr(pom)%0x100); //Vepsani adresy pro nespl.
      Start.sharp.program[(pom2)-2]=(byte)(Start.sharp.recountAdr(pom)/0x100);
    }
    return "";
  }
}

/*****************************************************************************/

class Seznam extends Prikaz {
  Prikaz prvni;
  Prikaz dalsi;

  public Seznam(Prikaz prvni, Prikaz dalsi) {
    this.prvni=prvni;
    this.dalsi=dalsi;
  }

  public Uzel optimizuj() {return this;}
  public String preloz() {
    prvni.preloz();
    dalsi.preloz();
    return "";
  }
}


